@@ -2,8 +2,9 @@ package Mojo::Pg::Migrations;
 use Mojo::Base -base;
 
 use Carp 'croak';
+use Mojo::File 'path';
 use Mojo::Loader 'data_section';
-use Mojo::Util qw(decode slurp);
+use Mojo::Util 'decode';
 
 use constant DEBUG => $ENV{MOJO_MIGRATIONS_DEBUG} || 0;
 
@@ -18,7 +19,7 @@ sub from_data {
     data_section($class //= caller, $name // $self->name));
 }
 
-sub from_file { shift->from_string(decode 'UTF-8', slurp pop) }
+sub from_file { shift->from_string(decode 'UTF-8', path(pop)->slurp) }
 
 sub from_string {
   my ($self, $sql) = @_;
@@ -59,19 +60,7 @@ sub migrate {
   croak "Active version $active is greater than the latest version $latest"
     if $active > $latest;
 
-  # Up
-  my $sql;
-  if ($active < $target) {
-    my @up = grep { $_ <= $target && $_ > $active } keys %$up;
-    $sql = join '', @$up{sort { $a <=> $b } @up};
-  }
-
-  # Down
-  else {
-    my @down = grep { $_ > $target && $_ <= $active } keys %$down;
-    $sql = join '', @$down{reverse sort { $a <=> $b } @down};
-  }
-
+  my $sql = $self->sql_for($active, $target);
   warn "-- Migrate ($active -> $target)\n$sql\n" if DEBUG;
   $sql .= ';update mojo_migrations set version = $1 where name = $2;';
   $db->query($sql, $target, $self->name) and $tx->commit;
@@ -79,6 +68,21 @@ sub migrate {
   return $self;
 }
 
+sub sql_for {
+  my ($self, $from, $to) = @_;
+
+  # Up
+  my ($up, $down) = @{$self->{migrations}}{qw(up down)};
+  if ($from < $to) {
+    my @up = grep { $_ <= $to && $_ > $from } keys %$up;
+    return join '', @$up{sort { $a <=> $b } @up};
+  }
+
+  # Down
+  my @down = grep { $_ > $to && $_ <= $from } keys %$down;
+  return join '', @$down{reverse sort { $a <=> $b } @down};
+}
+
 sub _active {
   my ($self, $db, $create) = @_;
 
@@ -224,6 +228,12 @@ representing an empty database.
   # Reset database
   $migrations->migrate(0)->migrate;
 
+=head2 sql_for
+
+  my $sql = $migrations->sql_for(5, 10);
+
+Get SQL to migrate from one version to another, up or down.
+
 =head1 DEBUGGING
 
 You can set the C<MOJO_MIGRATIONS_DEBUG> environment variable to get some
@@ -233,6 +243,6 @@ advanced diagnostics information printed to C<STDERR>.
 
 =head1 SEE ALSO
 
-L<Mojo::Pg>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
+L<Mojo::Pg>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
 
 =cut
