@@ -6,7 +6,7 @@ use Scalar::Util 'weaken';
 
 has 'pg';
 
-sub DESTROY { shift->_cleanup }
+sub DESTROY { Mojo::Util::_global_destruction() or shift->reset }
 
 sub json { ++$_[0]{json}{$_[1]} and return $_[0] }
 
@@ -19,6 +19,13 @@ sub listen {
 
 sub notify { $_[0]->_db->notify(_json(@_)) and return $_[0] }
 
+sub reset {
+  my $self = shift;
+  delete @$self{qw(chans json pid)};
+  return unless my $db = delete $self->{db};
+  ++$db->dbh->{private_mojo_no_reuse} and $db->_unwatch;
+}
+
 sub unlisten {
   my ($self, $name, $cb) = @_;
   my $chan = $self->{chans}{$name};
@@ -27,18 +34,9 @@ sub unlisten {
   return $self;
 }
 
-sub _cleanup {
-  my $self = shift;
-  $self->{db}->_unwatch;
-  delete @$self{qw(chans db pid)};
-}
-
 sub _db {
   my $self = shift;
 
-  # Fork-safety
-  $self->_cleanup unless ($self->{pid} //= $$) eq $$;
-
   return $self->{db} if $self->{db};
 
   my $db = $self->{db} = $self->pg->db;
@@ -48,15 +46,11 @@ sub _db {
     notification => sub {
       my ($db, $name, $pid, $payload) = @_;
       $payload = eval { from_json $payload } if $self->{json}{$name};
-      for my $cb (@{$self->{chans}{$name}}) { $self->$cb($payload) }
-    }
-  );
-  $db->once(
-    close => sub {
-      delete $self->{db};
-      eval { $self->_db };
+      my @cbs = @{$self->{chans}{$name}};
+      for my $cb (@cbs) { $self->$cb($payload) }
     }
   );
+  $db->once(close => sub { $self->{pg} and $self->_db if delete $self->{db} });
   $db->listen($_) for keys %{$self->{chans}}, 'mojo.pubsub';
   $self->emit(reconnect => $db);
 
@@ -92,10 +86,6 @@ pattern used by L<Mojo::Pg>. It is based on PostgreSQL notifications and allows
 many consumers to share the same database connection, to avoid many common
 scalability problems.
 
-All subscriptions will be reset automatically and the database connection
-re-established if a new process has been forked, this allows multiple processes
-to share the same L<Mojo::Pg::PubSub> object safely.
-
 =head1 EVENTS
 
 L<Mojo::Pg::PubSub> inherits all events from L<Mojo::EventEmitter> and can
@@ -168,6 +158,14 @@ L</"json">.
 Notify a channel. Automatic encoding of Perl values to JSON text can be
 activated with L</"json">.
 
+=head2 reset
+
+  $pubsub->reset;
+
+Reset all subscriptions and the database connection. This is usually done after
+a new process has been forked, to prevent the child process from stealing
+notifications meant for the parent process.
+
 =head2 unlisten
 
   $pubsub = $pubsub->unlisten('foo');
