@@ -1,7 +1,37 @@
 package SQL::Abstract::Pg;
 use Mojo::Base 'SQL::Abstract';
 
-use Carp 'croak';
+sub insert {
+  my ($self, $table, $data, $options) = @_;
+  local @{$options}{qw(returning _pg_returning)} = (1, 1)
+    if defined $options->{on_conflict} && !$options->{returning};
+  return $self->SUPER::insert($table, $data, $options);
+}
+
+sub _insert_returning {
+  my ($self, $options) = @_;
+
+  delete $options->{returning} if $options->{_pg_returning};
+
+  # ON CONFLICT
+  my $sql = '';
+  my @bind;
+  if (defined(my $conflict = $options->{on_conflict})) {
+    my ($conflict_sql, @conflict_bind);
+    $self->_SWITCH_refkind(
+      $conflict => {
+        SCALARREF => sub { $conflict_sql = $$conflict },
+        ARRAYREFREF => sub { ($conflict_sql, @conflict_bind) = @$$conflict }
+      }
+    );
+    $sql .= $self->_sqlcase(' on conflict ') . $conflict_sql;
+    push @bind, @conflict_bind;
+  }
+
+  $sql .= $self->SUPER::_insert_returning($options) if $options->{returning};
+
+  return $sql, @bind;
+}
 
 sub _order_by {
   my ($self, $arg) = @_;
@@ -11,25 +41,31 @@ sub _order_by {
     if ref $arg ne 'HASH'
     or grep {/^-(?:desc|asc)/i} keys %$arg;
 
-  return $self->_parse($arg);
+  return $self->_pg_parse($arg);
 }
 
-sub _parse {
+sub _pg_parse {
   my ($self, $options) = @_;
 
   # GROUP BY
   my $sql = '';
   my @bind;
-  if (defined $options->{group_by}) {
-    croak qq{Unsupported group_by value "$options->{group_by}"}
-      unless ref $options->{group_by} eq 'SCALAR';
-    $sql .= $self->_sqlcase(' group by ') . ${$options->{group_by}};
+  if (defined(my $group = $options->{group_by})) {
+    my $group_sql;
+    $self->_SWITCH_refkind(
+      $group => {
+        SCALARREF => sub { $group_sql = $$group },
+        ARRAYREF => sub {
+          $group_sql = join ', ', map { $self->_quote($_) } @$group;
+        }
+      }
+    );
+    $sql .= $self->_sqlcase(' group by ') . $group_sql;
   }
 
   # ORDER BY
-  if (defined $options->{order_by}) {
-    $sql .= $self->_order_by($options->{order_by});
-  }
+  $sql .= $self->_order_by($options->{order_by})
+    if defined $options->{order_by};
 
   # LIMIT
   if (defined $options->{limit}) {
@@ -44,10 +80,14 @@ sub _parse {
   }
 
   # FOR
-  if (defined $options->{for}) {
-    croak qq{Unsupported for value "$options->{for}"}
-      unless ref $options->{for} eq 'SCALAR';
-    $sql .= $self->_sqlcase(' for ') . ${$options->{for}};
+  if (defined(my $for = $options->{for})) {
+    my $for_sql;
+    $self->_SWITCH_refkind(
+      $for => {
+        SCALARREF => sub { $for_sql .= $$for }
+      }
+    );
+    $sql .= $self->_sqlcase(' for ') . $for_sql;
   }
 
   return $sql, @bind;
@@ -70,6 +110,30 @@ SQL::Abstract::Pg - PostgreSQL Magic
 L<SQL::Abstract::Pg> extends L<SQL::Abstract> with a few PostgreSQL features
 used by L<Mojo::Pg>.
 
+=head1 INSERT
+
+Additional C<INSERT> query features.
+
+=head2 ON CONFLICT
+
+The C<on_conflict> option can be used to generate C<INSERT> queries with
+C<ON CONFLICT> clauses. So far only scalar references to pass literal SQL and
+array reference references to pass literal SQL with bind values are supported.
+
+  # "insert into some_table (foo) values ('bar') on conflict do nothing"
+  $abstract->insert(
+    'some_table', {foo => 'bar'}, {on_conflict => \'do nothing'});
+
+This includes operations commonly referred to as C<upsert>.
+
+  # "insert into t (a) values ('b') on conflict (a) do update set a = 'c'"
+  $abstract->insert(
+    't', {a => 'b'}, {on_conflict => \['(a) do update set a = ?', 'c']});
+
+=head1 SELECT
+
+Additional C<SELECT> query features.
+
 =head2 ORDER BY
 
   $abstract->select($source, $fields, $where, $order);
@@ -99,8 +163,11 @@ with C<LIMIT> and C<OFFSET> clauses.
 =head2 GROUP BY
 
 The C<group_by> option can be used to generate C<SELECT> queries with
-C<GROUP BY> clauses. So far only scalar references to pass literal SQL are
-supported.
+C<GROUP BY> clauses. So far only array references to pass a list of fields and
+scalar references to pass literal SQL are supported.
+
+  # "select * from some_table group by foo, bar"
+  $abstract->select('some_table', undef, undef, {group_by => ['foo', 'bar']});
 
   # "select * from some_table group by foo, bar"
   $abstract->select('some_table', undef, undef, {group_by => \'foo, bar'});
